@@ -541,9 +541,11 @@ create_content (GrlMedia *content,
     str = g_strdup (g_file_info_get_display_name (info));
 
     /* Remove file extension */
-    extension = g_strrstr (str, ".");
-    if (extension) {
-      *extension = '\0';
+    if (!GRL_IS_MEDIA_BOX (media)) {
+      extension = g_strrstr (str, ".");
+      if (extension) {
+        *extension = '\0';
+      }
     }
 
     grl_media_set_title (media, str);
@@ -1250,7 +1252,6 @@ grl_filesystem_source_browse (GrlSource *source,
                           GRL_RESOLVE_FAST_ONLY,
                           FALSE,
                           bs->options);
-        remaining--;
         if (content) {
           bs->callback (source,
                         bs->operation_id,
