@@ -99,9 +99,6 @@ gboolean grl_flickr_plugin_init (GrlRegistry *registry,
 
 static const GList *grl_flickr_source_supported_keys (GrlSource *source);
 
-static GrlCaps * grl_flickr_source_get_caps (GrlSource *source,
-                                             GrlSupportedOps operation);
-
 static void grl_flickr_source_browse (GrlSource *source,
                                       GrlSourceBrowseSpec *bs);
 
@@ -192,6 +189,7 @@ grl_flickr_source_public_new (const gchar *flickr_api_key,
                          "source-id", PUBLIC_SOURCE_ID,
                          "source-name", PUBLIC_SOURCE_NAME,
                          "source-desc", PUBLIC_SOURCE_DESC,
+                         "supported-media", GRL_MEDIA_TYPE_IMAGE,
                          NULL);
   source->priv->flickr = g_flickr_new (flickr_api_key, flickr_secret, NULL);
 
@@ -219,7 +217,6 @@ grl_flickr_source_class_init (GrlFlickrSourceClass * klass)
   gobject_class->finalize = grl_flickr_source_finalize;
 
   source_class->supported_keys = grl_flickr_source_supported_keys;
-  source_class->get_caps = grl_flickr_source_get_caps;
   source_class->browse = grl_flickr_source_browse;
   source_class->resolve = grl_flickr_source_resolve;
   source_class->search = grl_flickr_source_search;
@@ -394,7 +391,6 @@ search_cb (GFlickr *f, GList *photolist, gpointer user_data)
 {
   GrlMedia *media;
   OperationData *od = (OperationData *) user_data;
-  gchar *media_type;
 
   /* Go to offset element */
   photolist = g_list_nth (photolist, od->offset);
@@ -412,12 +408,7 @@ search_cb (GFlickr *f, GList *photolist, gpointer user_data)
   }
 
   while (photolist && od->count) {
-    media_type = g_hash_table_lookup (photolist->data, "photo_media");
-    if (strcmp (media_type, "photo") == 0) {
-      media = grl_media_image_new ();
-    } else {
-      media = grl_media_video_new ();
-    }
+    media = grl_media_image_new ();
     update_media (media, photolist->data);
     od->callback (od->source,
                   od->operation_id,
@@ -767,15 +758,3 @@ grl_flickr_source_search (GrlSource *source,
                                od);
   }
 }
-
-static GrlCaps *
-grl_flickr_source_get_caps (GrlSource *source,
-                            GrlSupportedOps operation)
-{
-  static GrlCaps *caps = NULL;
-
-  if (caps == NULL)
-    caps = grl_caps_new ();
-
-  return caps;
-}
