@@ -100,9 +100,6 @@ gboolean grl_shoutcast_plugin_init (GrlRegistry *registry,
 
 static const GList *grl_shoutcast_source_supported_keys (GrlSource *source);
 
-static GrlCaps * grl_shoutcast_source_get_caps (GrlSource *source,
-                                                GrlSupportedOps operation);
-
 static void grl_shoutcast_source_resolve (GrlSource *source,
                                           GrlSourceResolveSpec *rs);
 
@@ -182,6 +179,7 @@ grl_shoutcast_source_new (const gchar *dev_key)
                           "source-id", SOURCE_ID,
                           "source-name", SOURCE_NAME,
                           "source-desc", SOURCE_DESC,
+                          "supported-media", GRL_MEDIA_TYPE_AUDIO,
                           NULL);
 
   source->priv->dev_key = g_strdup (dev_key);
@@ -199,7 +197,6 @@ grl_shoutcast_source_class_init (GrlShoutcastSourceClass * klass)
 
   source_class->cancel = grl_shoutcast_source_cancel;
   source_class->supported_keys = grl_shoutcast_source_supported_keys;
-  source_class->get_caps = grl_shoutcast_source_get_caps;
   source_class->resolve = grl_shoutcast_source_resolve;
   source_class->browse = grl_shoutcast_source_browse;
   source_class->search = grl_shoutcast_source_search;
@@ -808,15 +805,3 @@ grl_shoutcast_source_cancel (GrlSource *source, guint operation_id)
     op_data->cancelled = TRUE;
   }
 }
-
-static GrlCaps *
-grl_shoutcast_source_get_caps (GrlSource *source,
-                               GrlSupportedOps operation)
-{
-  static GrlCaps *caps = NULL;
-
-  if (caps == NULL)
-    caps = grl_caps_new ();
-
-  return caps;
-}
