@@ -25,6 +25,7 @@
 #endif
 
 #include <string.h>
+#include <stdlib.h>
 
 #include <grilo.h>
 #include <net/grl-net.h>
@@ -76,7 +77,13 @@ struct _ResolveClosure {
   gboolean slow;
 };
 
+struct _PendingRequest {
+  GrlTmdbRequest *request;
+  GAsyncReadyCallback callback;
+};
+
 typedef struct _ResolveClosure ResolveClosure;
+typedef struct _PendingRequest PendingRequest;
 
 static GrlTmdbSource *grl_tmdb_source_new (const char *api_key);
 
@@ -112,6 +119,8 @@ register_metadata_key (GrlRegistry *registry,
                        const char *blurb);
 
 static void resolve_closure_free (ResolveClosure *closure);
+static void resolve_slow_details (ResolveClosure *closure);
+
 /* =================== GrlTmdbMetadata Plugin  =============== */
 
 
@@ -227,6 +236,8 @@ grl_tmdb_source_init (GrlTmdbSource *self)
   self->priv->slow_keys = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   /* Fast keys */
+  g_hash_table_add (self->priv->supported_keys,
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_TITLE));
   g_hash_table_add (self->priv->supported_keys,
                     GRLKEYID_TO_POINTER (GRL_METADATA_KEY_THUMBNAIL));
   g_hash_table_add (self->priv->supported_keys,
@@ -263,6 +274,8 @@ grl_tmdb_source_init (GrlTmdbSource *self)
                     GRLKEYID_TO_POINTER (GRL_METADATA_KEY_PRODUCER));
   g_hash_table_add (self->priv->slow_keys,
                     GRLKEYID_TO_POINTER (GRL_METADATA_KEY_DIRECTOR));
+  g_hash_table_add (self->priv->slow_keys,
+                    GRLKEYID_TO_POINTER (GRL_METADATA_KEY_AUTHOR));
 
   /* The publication date is both available as fast key in the movie details,
    * but also as more detailed information as regional release date. To avoid
@@ -398,11 +411,71 @@ resolve_closure_callback (ResolveClosure *closure,
     g_error_free (error);
 }
 
+static void queue_request (ResolveClosure *closure,
+                           GrlTmdbRequest *request,
+                           GAsyncReadyCallback callback)
+{
+  PendingRequest *pending_request;
+
+  pending_request = g_slice_new0 (PendingRequest);
+  pending_request->request = request;
+  pending_request->callback = callback;
+
+  g_queue_push_tail (closure->pending_requests, pending_request);
+}
+
+static int run_pending_requests (ResolveClosure *closure,
+                                 int max_num_request)
+{
+  int num_requests = 0;
+  GList *it;
+
+  for (it = closure->pending_requests->head; it; it = it->next) {
+    if (num_requests >= max_num_request)
+      break;
+
+    PendingRequest *const pending_request = it->data;
+
+    grl_tmdb_request_run_async (pending_request->request,
+                                closure->self->priv->wc,
+                                pending_request->callback,
+                                NULL,
+                                closure);
+
+    ++num_requests;
+  }
+
+  return num_requests;
+}
+
+static void pending_request_free (gpointer data)
+{
+  PendingRequest *const pending_request = data;
+  g_object_unref (pending_request->request);
+  g_slice_free (PendingRequest, pending_request);
+}
+
+static void remove_request (ResolveClosure *closure,
+                            GrlTmdbRequest *request)
+{
+  GList *it;
+
+  for (it = closure->pending_requests->head; it; it = it->next) {
+    PendingRequest *const pending_request = it->data;
+
+    if (pending_request->request == request) {
+      g_queue_unlink (closure->pending_requests, it);
+      pending_request_free (pending_request);
+      break;
+    }
+  }
+}
+
 static void
 resolve_closure_free (ResolveClosure *closure)
 {
   g_object_unref (closure->self);
-  g_queue_free_full (closure->pending_requests, g_object_unref);
+  g_queue_free_full (closure->pending_requests, pending_request_free);
   g_hash_table_destroy (closure->keys);
   g_slice_free (ResolveClosure, closure);
 }
@@ -447,6 +520,26 @@ director_filter (JsonNode *element)
   return g_strdup (json_object_get_string_member (object, "name"));
 }
 
+static char *
+writer_filter (JsonNode *element)
+{
+  JsonObject *object;
+  const char *department;
+
+  if (!JSON_NODE_HOLDS_OBJECT (element)) {
+    return NULL;
+  }
+
+  object = json_node_get_object (element);
+
+  department = json_object_get_string_member (object, "department");
+  if (g_ascii_strcasecmp (department, "Writing") != 0) {
+    return NULL;
+  }
+
+  return g_strdup (json_object_get_string_member (object, "name"));
+}
+
 static GDateTime *
 parse_date (const gchar *iso8601_string)
 {
@@ -488,17 +581,30 @@ add_image (GrlTmdbSource *self,
            GrlKeyID       detail_key,
            const char    *image_path)
 {
-    SoupURI *uri;
-    char *str;
+  SoupURI *uri;
+  GrlRelatedKeys *related_keys;
+  char *str;
+  int i, l;
 
-    str = g_strconcat ("original", image_path, NULL);
-    uri = soup_uri_new_with_base (self->priv->image_base_uri, str);
-    g_free (str);
+  str = g_strconcat ("original", image_path, NULL);
+  uri = soup_uri_new_with_base (self->priv->image_base_uri, str);
+  g_free (str);
 
-    str = soup_uri_to_string (uri, FALSE);
-    grl_media_add_thumbnail (media, str);
+  str = soup_uri_to_string (uri, FALSE);
+
+  l = grl_data_length (GRL_DATA (media), detail_key);
+
+  for (i = 0; i < l; ++i) {
+    related_keys = grl_data_get_related_keys (GRL_DATA (media), detail_key, i);
+    if (g_strcmp0 (grl_related_keys_get_string (related_keys, detail_key), str) == 0)
+      break;
+  }
+
+  if (i == l) {
     grl_data_add_string (GRL_DATA (media), detail_key, str);
-    g_free (str);
+  }
+
+  g_free (str);
 }
 
 static void
@@ -507,11 +613,14 @@ on_request_ready (GObject *source,
                   gpointer user_data) {
   ResolveClosure *closure = (ResolveClosure *) user_data;
   GrlTmdbRequest *request = GRL_TMDB_REQUEST (source);
+  const GrlTmdbRequestDetail detail = grl_tmdb_request_get_detail (request);
   GError *error = NULL;
   GList *values, *iter;
   GValue *value;
 
-  GRL_DEBUG ("Detail request ready...");
+  GRL_DEBUG ("Detail request (%s) ready for movie #%" G_GUINT64_FORMAT "...",
+             grl_tmdb_request_detail_to_string (detail), closure->id);
+
   if (!grl_tmdb_request_run_finish (GRL_TMDB_REQUEST (source),
                                     result,
                                     &error)) {
@@ -522,7 +631,7 @@ on_request_ready (GObject *source,
     goto out;
   }
 
-  switch (grl_tmdb_request_get_detail (request)) {
+  switch (detail) {
     case GRL_TMDB_REQUEST_DETAIL_MOVIE:
     {
       if (SHOULD_RESOLVE (GRL_METADATA_KEY_GENRE)) {
@@ -571,13 +680,92 @@ on_request_ready (GObject *source,
           g_value_unset (value);
         }
       }
+
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_RATING)) {
+        value = grl_tmdb_request_get (request, "$.vote_average");
+        if (value != NULL) {
+          grl_media_set_rating (closure->rs->media,
+                                (float) g_value_get_double (value),
+                                10.0f);
+          g_value_unset (value);
+        }
+      }
+
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_ORIGINAL_TITLE)) {
+        value = grl_tmdb_request_get (request, "$.original_title");
+        if (value != NULL) {
+          grl_media_video_set_original_title (GRL_MEDIA_VIDEO (closure->rs->media),
+                                              g_value_get_string (value));
+          g_value_unset (value);
+        }
+      }
+
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_TITLE)) {
+        value = grl_tmdb_request_get (request, "$.title");
+        if (value != NULL) {
+          grl_media_set_title (closure->rs->media, g_value_get_string (value));
+          g_value_unset (value);
+        }
+      }
+
+      if (!closure->slow) {
+        /* Add thumbnails first and poster and backdrops later.
+         * Posters more likely make a good thumbnail than backdrops.
+         */
+        if (SHOULD_RESOLVE (GRL_METADATA_KEY_THUMBNAIL)) {
+          value = grl_tmdb_request_get (request, "$.poster_path");
+          if (value != NULL) {
+              add_image (closure->self, closure->rs->media,
+                         GRL_METADATA_KEY_THUMBNAIL,
+                         g_value_get_string (value));
+
+              g_value_unset (value);
+          }
+        }
+
+        if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER)) {
+          value = grl_tmdb_request_get (request, "$.poster_path");
+          if (value != NULL) {
+              add_image (closure->self, closure->rs->media,
+                         GRL_TMDB_METADATA_KEY_POSTER,
+                         g_value_get_string (value));
+
+              g_value_unset (value);
+          }
+        }
+
+        if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROP)) {
+          value = grl_tmdb_request_get (request, "$.backdrop_path");
+          if (value != NULL) {
+            add_image (closure->self, closure->rs->media,
+                       GRL_TMDB_METADATA_KEY_BACKDROP,
+                       g_value_get_string (value));
+
+            g_value_unset (value);
+          }
+        }
+      }
     }
     break;
     case GRL_TMDB_REQUEST_DETAIL_MOVIE_IMAGES:
     {
-      /* Add posters first and backdrops later.
+      /* Add thumbnails first, and posters and backdrops later.
        * Posters more likely make a good thumbnail than backdrops.
        */
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_THUMBNAIL)) {
+        iter = values = grl_tmdb_request_get_string_list_with_filter (request,
+                                                                      "$.posters",
+                                                                      neutral_backdrop_filter);
+        while (iter != NULL) {
+          add_image (closure->self, closure->rs->media,
+                     GRL_METADATA_KEY_THUMBNAIL,
+                     iter->data);
+
+          iter = iter->next;
+        }
+        g_list_free_full (values, g_free);
+      }
+
       if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER)) {
         iter = values = grl_tmdb_request_get_string_list_with_filter (request,
                                                                       "$.posters",
@@ -658,6 +846,19 @@ on_request_ready (GObject *source,
         }
         g_list_free_full (values, g_free);
       }
+
+      if (SHOULD_RESOLVE (GRL_METADATA_KEY_AUTHOR)) {
+        values = grl_tmdb_request_get_string_list_with_filter (request,
+                                                               "$.crew[*]",
+                                                               writer_filter);
+        iter = values;
+        while (iter != NULL) {
+          grl_media_add_author (GRL_MEDIA (closure->rs->media),
+                                iter->data);
+          iter = iter->next;
+        }
+        g_list_free_full (values, g_free);
+      }
     }
     break;
     case GRL_TMDB_REQUEST_DETAIL_MOVIE_RELEASE_INFO:
@@ -685,7 +886,7 @@ on_request_ready (GObject *source,
           g_date_time_unref (pubdate);
         }
 
-        g_list_free (values);
+        g_list_free_full (values, (GDestroyNotify) json_node_free);
       }
     }
     break;
@@ -697,8 +898,8 @@ out:
   if (error != NULL) {
     g_error_free (error);
   }
-  g_queue_remove (closure->pending_requests, request);
-  g_object_unref (request);
+
+  remove_request (closure, request);
 
   if (g_queue_is_empty (closure->pending_requests)) {
     resolve_closure_callback (closure, NULL);
@@ -706,25 +907,6 @@ out:
   }
 }
 
-static GrlTmdbRequest *
-create_and_run_request (GrlTmdbSource *self,
-                        ResolveClosure *closure,
-                        GrlTmdbRequestDetail id)
-{
-  GrlTmdbRequest *request;
-
-  request = grl_tmdb_request_new_details (self->priv->api_key,
-                                          id,
-                                          closure->id);
-  grl_tmdb_request_run_async (request,
-                              self->priv->wc,
-                              on_request_ready,
-                              NULL,
-                              closure);
-
-  return request;
-}
-
 static void
 on_search_ready (GObject *source,
                  GAsyncResult *result,
@@ -732,7 +914,6 @@ on_search_ready (GObject *source,
 {
   ResolveClosure *closure = (ResolveClosure *) user_data;
   GrlTmdbRequest *request = GRL_TMDB_REQUEST (source);
-  GrlTmdbSource *self = closure->self;
   GValue *value;
   GError *error = NULL;
 
@@ -785,11 +966,23 @@ on_search_ready (GObject *source,
                             10.0f);
       g_value_unset (value);
     }
+    g_hash_table_remove (closure->keys, GRLKEYID_TO_POINTER (GRL_METADATA_KEY_RATING));
   }
 
-  /* Add posters first and backdrops later.
+  /* Add thumbnails first, and posters and backdrops later.
    * Posters more likely make a good thumbnail than backdrops.
    */
+  if (SHOULD_RESOLVE (GRL_METADATA_KEY_THUMBNAIL)) {
+    value = grl_tmdb_request_get (request, "$.results[0].poster_path");
+    if (value != NULL) {
+        add_image (closure->self, closure->rs->media,
+                   GRL_METADATA_KEY_THUMBNAIL,
+                   g_value_get_string (value));
+
+        g_value_unset (value);
+    }
+  }
+
   if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER)) {
     value = grl_tmdb_request_get (request, "$.results[0].poster_path");
     if (value != NULL) {
@@ -798,7 +991,6 @@ on_search_ready (GObject *source,
                    g_value_get_string (value));
 
         g_value_unset (value);
-
     }
   }
 
@@ -813,15 +1005,6 @@ on_search_ready (GObject *source,
     }
   }
 
-  if (SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE)) {
-    value = grl_tmdb_request_get (request, "$.results[0].adult");
-    if (value != NULL) {
-      if (g_value_get_boolean (value))
-        grl_media_set_certificate (closure->rs->media, "adult");
-      g_value_unset (value);
-    }
-  }
-
   if (SHOULD_RESOLVE (GRL_METADATA_KEY_ORIGINAL_TITLE)) {
     value = grl_tmdb_request_get (request, "$.results[0].original_title");
     if (value != NULL) {
@@ -829,58 +1012,66 @@ on_search_ready (GObject *source,
                                           g_value_get_string (value));
       g_value_unset (value);
     }
+    g_hash_table_remove (closure->keys, GRLKEYID_TO_POINTER (GRL_METADATA_KEY_ORIGINAL_TITLE));
   }
 
-  g_queue_pop_head (closure->pending_requests);
-  g_object_unref (source);
+  remove_request (closure, request);
 
-  /* No need to do additional requests */
-  if (!closure->slow) {
-    resolve_closure_callback (closure, NULL);
-    resolve_closure_free (closure);
-    return;
+  /* Check if we need to do additional requests. */
+  if (closure->slow) {
+    resolve_slow_details (closure);
+
+    if (run_pending_requests (closure, G_MAXINT) > 0)
+      return;
   }
 
-  /* We need to do additional requests. Check if we should have resolved
-   * images and try to get some more */
+  resolve_closure_callback (closure, NULL);
+  resolve_closure_free (closure);
+}
+
+static void queue_detail_request (ResolveClosure *closure,
+                                  GrlTmdbRequestDetail detail)
+{
+  GrlTmdbRequest *request;
+
+  GRL_DEBUG ("Requesting %s for movie #%" G_GUINT64_FORMAT "...",
+             grl_tmdb_request_detail_to_string (detail), closure->id);
+
+  request = grl_tmdb_request_new_details (closure->self->priv->api_key,
+                                          detail, closure->id);
+
+  queue_request (closure, request, on_request_ready);
+}
+
+static void resolve_slow_details (ResolveClosure *closure)
+{
   if (SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_BACKDROP) ||
       SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_POSTER))
-    g_queue_push_tail (closure->pending_requests,
-                       create_and_run_request (self,
-                                               closure,
-                                               GRL_TMDB_REQUEST_DETAIL_MOVIE_IMAGES));
+    queue_detail_request (closure, GRL_TMDB_REQUEST_DETAIL_MOVIE_IMAGES);
 
-  if (SHOULD_RESOLVE (GRL_METADATA_KEY_GENRE) ||
+  if (SHOULD_RESOLVE (GRL_METADATA_KEY_RATING) ||
+      SHOULD_RESOLVE (GRL_METADATA_KEY_ORIGINAL_TITLE) ||
+      SHOULD_RESOLVE (GRL_METADATA_KEY_TITLE) ||
+      SHOULD_RESOLVE (GRL_METADATA_KEY_GENRE) ||
       SHOULD_RESOLVE (GRL_METADATA_KEY_STUDIO) ||
       SHOULD_RESOLVE (GRL_METADATA_KEY_SITE) ||
       SHOULD_RESOLVE (GRL_METADATA_KEY_DESCRIPTION) ||
       SHOULD_RESOLVE (GRL_TMDB_METADATA_KEY_IMDB_ID))
-    g_queue_push_tail (closure->pending_requests,
-                       create_and_run_request (self,
-                                               closure,
-                                               GRL_TMDB_REQUEST_DETAIL_MOVIE));
+    queue_detail_request (closure, GRL_TMDB_REQUEST_DETAIL_MOVIE);
 
   if (SHOULD_RESOLVE (GRL_METADATA_KEY_KEYWORD))
-    g_queue_push_tail (closure->pending_requests,
-                       create_and_run_request (self,
-                                               closure,
-                                               GRL_TMDB_REQUEST_DETAIL_MOVIE_KEYWORDS));
+    queue_detail_request (closure, GRL_TMDB_REQUEST_DETAIL_MOVIE_KEYWORDS);
 
   if (SHOULD_RESOLVE (GRL_METADATA_KEY_PERFORMER) ||
       SHOULD_RESOLVE (GRL_METADATA_KEY_PRODUCER) ||
-      SHOULD_RESOLVE (GRL_METADATA_KEY_DIRECTOR))
-    g_queue_push_tail (closure->pending_requests,
-                       create_and_run_request (self,
-                                               closure,
-                                               GRL_TMDB_REQUEST_DETAIL_MOVIE_CAST));
+      SHOULD_RESOLVE (GRL_METADATA_KEY_DIRECTOR) ||
+      SHOULD_RESOLVE (GRL_METADATA_KEY_AUTHOR))
+    queue_detail_request (closure, GRL_TMDB_REQUEST_DETAIL_MOVIE_CAST);
 
   if (SHOULD_RESOLVE (GRL_METADATA_KEY_REGION) ||
           SHOULD_RESOLVE (GRL_METADATA_KEY_CERTIFICATE) ||
           SHOULD_RESOLVE (GRL_METADATA_KEY_PUBLICATION_DATE))
-    g_queue_push_tail (closure->pending_requests,
-                       create_and_run_request (self,
-                                               closure,
-                                               GRL_TMDB_REQUEST_DETAIL_MOVIE_RELEASE_INFO));
+    queue_detail_request (closure, GRL_TMDB_REQUEST_DETAIL_MOVIE_RELEASE_INFO);
 }
 
 static void
@@ -894,6 +1085,8 @@ on_configuration_ready (GObject *source,
   GError *error = NULL;
   GValue *value;
 
+  GRL_DEBUG ("Configuration request ready...");
+
   if (!grl_tmdb_request_run_finish (GRL_TMDB_REQUEST (source),
                                     result,
                                     &error)) {
@@ -914,7 +1107,8 @@ on_configuration_ready (GObject *source,
     return;
   }
 
-  self->priv->configuration = g_queue_pop_head (closure->pending_requests);
+  self->priv->configuration = g_object_ref (request);
+  remove_request (closure, request);
 
   value = grl_tmdb_request_get (request, "$.images.base_url");
   if (value != NULL) {
@@ -929,11 +1123,7 @@ on_configuration_ready (GObject *source,
     ResolveClosure *pending_closure;
 
     pending_closure = g_queue_pop_head (self->priv->pending_resolves);
-    grl_tmdb_request_run_async (g_queue_peek_head (pending_closure->pending_requests),
-                                self->priv->wc,
-                                on_search_ready,
-                                NULL,
-                                pending_closure);
+    run_pending_requests (pending_closure, G_MAXINT);
   }
 }
 
@@ -999,8 +1189,9 @@ grl_tmdb_source_may_resolve (GrlSource *source,
     return FALSE;
   }
 
-  /* We can do nothing without a title */
-  if (!grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_TITLE)) {
+  /* We can do nothing without a title or the movie-id */
+  if (!grl_data_has_key (GRL_DATA (media), GRL_METADATA_KEY_TITLE) &&
+          !grl_data_has_key (GRL_DATA (media), GRL_TMDB_METADATA_KEY_TMDB_ID)) {
     if (!missing_keys)
       *missing_keys = grl_metadata_key_list_new (GRL_METADATA_KEY_TITLE, NULL);
 
@@ -1016,9 +1207,10 @@ grl_tmdb_source_resolve (GrlSource *source,
 {
   ResolveClosure *closure;
   GrlTmdbRequest *request;
-  GAsyncReadyCallback callback;
-  const char *title;
+  const char *title = NULL;
+  const char *str_movie_id;
   GrlTmdbSource *self = GRL_TMDB_SOURCE (source);
+  guint64 movie_id = 0;
   GList *it;
 
   if (!GRL_IS_MEDIA_VIDEO (rs->media)) {
@@ -1027,9 +1219,19 @@ grl_tmdb_source_resolve (GrlSource *source,
     return;
   }
 
-  title = grl_media_get_title (rs->media);
-  if (title == NULL) {
-    /* Can't search for anything without a title ... */
+  /* Prefer resolving by movie-id: This is more reliable and saves the search query. */
+  str_movie_id = grl_data_get_string (GRL_DATA (rs->media),
+                                      GRL_TMDB_METADATA_KEY_TMDB_ID);
+
+  if (str_movie_id)
+    movie_id = strtoull (str_movie_id, NULL, 10);
+
+  /* Try title if no movie-id could be found. */
+  if (movie_id == 0)
+    title = grl_media_get_title (rs->media);
+
+  if (movie_id == 0 && title == NULL) {
+    /* Can't search for anything without a title or the movie-id ... */
     rs->callback (source, rs->operation_id, rs->media, rs->user_data, NULL);
     return;
   }
@@ -1042,8 +1244,8 @@ grl_tmdb_source_resolve (GrlSource *source,
   closure->pending_requests = g_queue_new ();
   closure->keys = g_hash_table_new (g_direct_hash, g_direct_equal);
   closure->slow = FALSE;
-  request = grl_tmdb_request_new_search (closure->self->priv->api_key, title);
-  g_queue_push_tail (closure->pending_requests, request);
+  closure->id = movie_id;
+
   it = rs->keys;
 
   /* Copy keys to list for faster lookup */
@@ -1073,17 +1275,26 @@ grl_tmdb_source_resolve (GrlSource *source,
     GRL_DEBUG ("Fetching TMDb configuration...");
     /* We need to fetch TMDb's configuration for the image paths */
     request = grl_tmdb_request_new_configuration (closure->self->priv->api_key);
-    g_queue_push_head (closure->pending_requests, request);
-    callback = on_configuration_ready;
+    g_assert (g_queue_is_empty (closure->pending_requests));
+    queue_request (closure, request, on_configuration_ready);
     self->priv->config_pending = TRUE;
-  } else {
-    GRL_DEBUG ("Running initial search...");
-    callback = on_search_ready;
   }
 
-  grl_tmdb_request_run_async (g_queue_peek_head (closure->pending_requests),
-                              closure->self->priv->wc,
-                              callback,
-                              NULL,
-                              closure);
+  if (title) {
+    GRL_DEBUG ("Running initial search for title \"%s\"...", title);
+    request = grl_tmdb_request_new_search (closure->self->priv->api_key, title);
+    queue_request (closure, request, on_search_ready);
+    run_pending_requests (closure, 1);
+  } else {
+    GRL_DEBUG ("Running %s lookup for movie #%" G_GUINT64_FORMAT "...",
+               closure->slow ? "slow" : "fast", movie_id);
+
+    if (closure->slow) {
+      resolve_slow_details (closure);
+    } else {
+      queue_detail_request (closure, GRL_TMDB_REQUEST_DETAIL_MOVIE);
+    }
+
+    run_pending_requests (closure, G_MAXINT);
+  }
 }
