@@ -58,19 +58,34 @@ GRL_LOG_DOMAIN_STATIC(upnp_log_domain);
 #endif
 
 #define UPNP_SEARCH_SPEC                        \
-  "upnp:class derivedfrom \"object.item\" and " \
+  "%s and "                                     \
   "(dc:title contains \"%s\" or "               \
   "upnp:album contains \"%s\" or "              \
   "upnp:artist contains \"%s\")"
 
-#define UPNP_SEARCH_ALL                         \
+#define UPNP_BROWSE_SPEC                                 \
+  "( %s or "                                             \
+  "upnp:class derivedfrom \"object.container\" ) and "   \
+  "@parentID = \"%s\""                                   \
+
+#define UPNP_TYPE_FILTER_ALL                    \
   "upnp:class derivedfrom \"object.item\""
 
+#define UPNP_TYPE_FILTER_AUDIO                        \
+  "upnp:class derivedfrom \"object.item.audioItem\""
+
+#define UPNP_TYPE_FILTER_VIDEO                        \
+  "upnp:class derivedfrom \"object.item.videoItem\""
+
+#define UPNP_TYPE_FILTER_IMAGE                        \
+  "upnp:class derivedfrom \"object.item.imageItem\""
+
 struct _GrlUpnpPrivate {
   GUPnPDeviceProxy* device;
   GUPnPServiceProxy* service;
   GUPnPControlPoint *cp;
   gboolean search_enabled;
+  gboolean browse_filtered_enabled;
   gchar *upnp_name;
 };
 
@@ -117,6 +132,9 @@ static void grl_upnp_source_query (GrlSource *source,
 static void grl_upnp_source_resolve (GrlSource *source,
                                      GrlSourceResolveSpec *rs);
 
+static GrlCaps *grl_upnp_source_get_caps (GrlSource *source,
+                                          GrlSupportedOps operation);
+
 static gboolean grl_upnp_source_notify_change_start (GrlSource *source,
                                                      GError **error);
 
@@ -222,6 +240,7 @@ grl_upnp_source_class_init (GrlUpnpSourceClass * klass)
   source_class->search = grl_upnp_source_search;
   source_class->query = grl_upnp_source_query;
   source_class->resolve = grl_upnp_source_resolve;
+  source_class->get_caps = grl_upnp_source_get_caps;
   source_class->notify_change_start = grl_upnp_source_notify_change_start;
   source_class->notify_change_stop = grl_upnp_source_notify_change_stop;
 
@@ -332,12 +351,31 @@ gupnp_search_caps_cb (GUPnPServiceProxy *service,
 
   source = GRL_SOURCE (source_info->source);
 
-  if (caps && caps[0] != '\0') {
+  if (caps) {
+    if (g_strcmp0 (caps, "*") == 0) {
+      source_info->source->priv->search_enabled = TRUE;
+      source_info->source->priv->browse_filtered_enabled = TRUE;
+    } else {
+      if (g_strstr_len (caps, -1, "upnp:class")) {
+        if (g_strstr_len (caps, -1, "dc:title") ||
+            g_strstr_len (caps, -1, "upnp:album") ||
+            g_strstr_len (caps, -1, "upnp_artist")) {
+          source_info->source->priv->search_enabled = TRUE;
+        }
+        if (g_strstr_len (caps, -1, "@parentID")) {
+          source_info->source->priv->browse_filtered_enabled = TRUE;
+        }
+      }
+    }
+  }
+
+  if (source_info->source->priv->search_enabled) {
     GRL_DEBUG ("Setting search enabled for source '%s'",
                grl_source_get_name (source));
-    source_info->source->priv->search_enabled = TRUE;
-  } else {
-    GRL_DEBUG ("Setting search disabled for source '%s'",
+  }
+
+  if (source_info->source->priv->browse_filtered_enabled) {
+    GRL_DEBUG ("Setting filtered browse enabled for source '%s'",
                grl_source_get_name (source));
   }
 
@@ -530,13 +568,72 @@ get_upnp_filter (const GList *keys)
 }
 
 static gchar *
-get_upnp_search (const gchar *text)
+get_upnp_type_filter (GrlTypeFilter type_filter)
 {
-  if (text) {
-    return g_strdup_printf (UPNP_SEARCH_SPEC, text, text, text);
-  } else {
-    return g_strdup (UPNP_SEARCH_ALL);
+  GString *filter;
+  gboolean append_or = FALSE;
+
+  if (type_filter == GRL_TYPE_FILTER_ALL) {
+    return g_strdup (UPNP_TYPE_FILTER_ALL);
   }
+
+  filter = g_string_new ("( ");
+
+  if (type_filter & GRL_TYPE_FILTER_AUDIO) {
+    filter = g_string_append (filter, UPNP_TYPE_FILTER_AUDIO);
+    append_or = TRUE;
+  }
+
+  if (type_filter & GRL_TYPE_FILTER_VIDEO) {
+    if (append_or) {
+      filter = g_string_append (filter, " or ");
+    }
+    filter = g_string_append (filter, UPNP_TYPE_FILTER_VIDEO);
+    append_or = TRUE;
+  }
+
+  if (type_filter & GRL_TYPE_FILTER_IMAGE) {
+    if (append_or) {
+      filter = g_string_append (filter, " or ");
+    }
+    filter = g_string_append (filter, UPNP_TYPE_FILTER_IMAGE);
+  }
+
+  filter = g_string_append (filter, " )");
+
+  return g_string_free (filter, FALSE);
+}
+
+static gchar *
+get_upnp_search (GrlTypeFilter type_filter, const gchar *text)
+{
+  gchar *type_filter_string;
+  gchar *full_filter;
+
+  type_filter_string = get_upnp_type_filter (type_filter);
+
+  if (text) {
+    full_filter = g_strdup_printf (UPNP_SEARCH_SPEC, type_filter_string, text, text, text);
+    g_free (type_filter_string);
+  } else {
+    full_filter = type_filter_string;
+  }
+
+  return full_filter;
+}
+
+
+static gchar *
+get_upnp_browse (GrlTypeFilter type_filter, const gchar *container_id)
+{
+  gchar *type_filter_string;
+  gchar *full_filter;
+
+  type_filter_string = get_upnp_type_filter (type_filter);
+  full_filter = g_strdup_printf (UPNP_BROWSE_SPEC, type_filter_string, container_id);
+  g_free (type_filter_string);
+
+  return full_filter;
 }
 
 static void
@@ -1087,7 +1184,9 @@ grl_upnp_source_browse (GrlSource *source,
                         GrlSourceBrowseSpec *bs)
 {
   GUPnPServiceProxyAction* action;
+  GrlTypeFilter filter;
   gchar *upnp_filter;
+  gchar *upnp_browse;
   gchar *container_id;
   GError *error = NULL;
   struct OperationSpec *os;
@@ -1097,6 +1196,7 @@ grl_upnp_source_browse (GrlSource *source,
   upnp_filter = get_upnp_filter (bs->keys);
   GRL_DEBUG ("filter: '%s'", upnp_filter);
 
+
   os = g_slice_new0 (struct OperationSpec);
   os->source = bs->source;
   os->operation_id = bs->operation_id;
@@ -1111,23 +1211,38 @@ grl_upnp_source_browse (GrlSource *source,
     container_id = "0";
   }
 
-  action =
-    gupnp_service_proxy_begin_action (GRL_UPNP_SOURCE (source)->priv->service,
-				      "Browse", gupnp_browse_cb,
-                                      os,
-				      "ObjectID", G_TYPE_STRING,
-                                      container_id,
-				      "BrowseFlag", G_TYPE_STRING,
-                                      "BrowseDirectChildren",
-				      "Filter", G_TYPE_STRING,
-                                      upnp_filter,
-				      "StartingIndex", G_TYPE_UINT,
-                                      os->skip,
-				      "RequestedCount", G_TYPE_UINT,
-                                      os->count,
-				      "SortCriteria", G_TYPE_STRING,
-                                      "",
-				      NULL);
+  /* Check if we need to use underlaying search or browse */
+  filter = grl_operation_options_get_type_filter (bs->options);
+  if (filter != GRL_TYPE_FILTER_ALL) {
+    upnp_browse = get_upnp_browse (filter, container_id);
+  } else {
+    upnp_browse = NULL;
+  }
+
+  if (upnp_browse) {
+    action =
+      gupnp_service_proxy_begin_action (GRL_UPNP_SOURCE (source)->priv->service,
+                                        "Search", gupnp_browse_cb, os,
+                                        "ContainerID", G_TYPE_STRING, container_id,
+                                        "SearchCriteria", G_TYPE_STRING, upnp_browse,
+                                        "Filter", G_TYPE_STRING, upnp_filter,
+                                        "StartingIndex", G_TYPE_UINT, os->skip,
+                                        "RequestedCount", G_TYPE_UINT, os->count,
+                                        "SortCriteria", G_TYPE_STRING, "",
+                                        NULL);
+  } else {
+    action =
+      gupnp_service_proxy_begin_action (GRL_UPNP_SOURCE (source)->priv->service,
+                                        "Browse", gupnp_browse_cb, os,
+                                        "ObjectID", G_TYPE_STRING, container_id,
+                                        "BrowseFlag", G_TYPE_STRING, "BrowseDirectChildren",
+                                        "Filter", G_TYPE_STRING, upnp_filter,
+                                        "StartingIndex", G_TYPE_UINT, os->skip,
+                                        "RequestedCount", G_TYPE_UINT, os->count,
+                                        "SortCriteria", G_TYPE_STRING, "",
+                                        NULL);
+  }
+
   if (!action) {
     error = g_error_new (GRL_CORE_ERROR,
 			 GRL_CORE_ERROR_BROWSE_FAILED,
@@ -1138,6 +1253,7 @@ grl_upnp_source_browse (GrlSource *source,
   }
 
   g_free (upnp_filter);
+  g_free (upnp_browse);
 }
 
 static void
@@ -1154,7 +1270,8 @@ grl_upnp_source_search (GrlSource *source, GrlSourceSearchSpec *ss)
   upnp_filter = get_upnp_filter (ss->keys);
   GRL_DEBUG ("filter: '%s'", upnp_filter);
 
-  upnp_search = get_upnp_search (ss->text);
+  upnp_search = get_upnp_search (grl_operation_options_get_type_filter (ss->options),
+                                 ss->text);
   GRL_DEBUG ("search: '%s'", upnp_search);
 
   os = g_slice_new0 (struct OperationSpec);
@@ -1325,6 +1442,35 @@ grl_upnp_source_supported_operations (GrlSource *source)
   return caps;
 }
 
+static GrlCaps *
+grl_upnp_source_get_caps (GrlSource *source,
+                          GrlSupportedOps operation)
+{
+  static GrlCaps *search_caps = NULL;
+  static GrlCaps *browse_caps = NULL;
+
+  if (!search_caps) {
+    search_caps = grl_caps_new ();
+    if (GRL_UPNP_SOURCE (source)->priv->search_enabled) {
+      grl_caps_set_type_filter (search_caps, GRL_TYPE_FILTER_ALL);
+    }
+  }
+
+  if (!browse_caps) {
+    browse_caps = grl_caps_new ();
+    if (GRL_UPNP_SOURCE (source)->priv->browse_filtered_enabled) {
+      grl_caps_set_type_filter (browse_caps, GRL_TYPE_FILTER_ALL);
+    }
+  }
+
+  if (operation == GRL_OP_BROWSE) {
+    return browse_caps;
+  } else {
+    return search_caps;
+  }
+}
+
+
 static gboolean
 grl_upnp_source_notify_change_start (GrlSource *source,
                                      GError **error)
